@@ -21,16 +21,16 @@
 #
 from __future__ import print_function
 
+import json
 import logging
 import os
 import sys
 
-import msgpack
-import json
 from tornado.options import define, options
+from tornado import httpclient
 from tornado.options import parse_command_line, parse_config_file
-
-from cocaine.services import Service
+from tornado.ioloop import IOLoop
+from tornado.httpclient import HTTPRequest
 
 log = logging.getLogger()
 
@@ -40,7 +40,7 @@ ActionGroup = "Actions"
 
 POSSIBLE_ACTIONS = ('upload', 'deploy', 'start', 'stop',
                     'listing', 'undeploy', 'info', 'remove')
-API_VERSION = 1.2
+API_VERSION = 1.3
 
 
 class Dispatcher(object):
@@ -58,19 +58,32 @@ class Dispatcher(object):
         return cls.dispatch_tree[action]()
 
 
-class Printer(object):
-    def __init__(self, print_func=None):
-        self._prev = ''
-        if print_func is not None:
-            self._print_func = print_func
+def printer(data):
+    print(data, end="" if data.endswith('\r') else "\n",
+          file=sys.stderr)
 
-    def _print_func(self, value):
-        print(value.strip('\n'))
 
-    def log(self, value):
-        if value != self._prev:
-            self._prev = str(value)
-            self._print_func(self._prev)
+def run_request(req):
+    IOLoop.instance().run_sync(lambda: client.fetch(req))
+
+
+def gen_url(url):
+    return "http://%s:%d/%s" % (options.host, options.port, url)
+
+
+def default_format_reply(body):
+    printer(body)
+
+
+def jail(f):
+    def wrapper(*args, **kwargs):
+        try:
+            return f(*args, **kwargs)
+        except Exception as err:
+            log.error(err)
+            exit(1)
+    return wrapper
+
 
 # AppGroup
 define('appname', group=AppGroup,
@@ -89,6 +102,10 @@ define('profile', group=AppGroup,
        help='profile to deploy with',
        type=str, default=None)
 
+define('manifest', group=AppGroup,
+       help='specify manifest path, which would be used',
+       type=str, default="")
+
 define('weight', group=AppGroup,
        help='application weight in group',
        type=int, default=0)
@@ -108,7 +125,7 @@ define('host', group=CocaineGroup,
 
 define('port', group=CocaineGroup,
        help='Port of buildfarm locator',
-       type=int, default=10053)
+       type=int, default=80)
 
 # ActionGroup
 define('action', group=ActionGroup,
@@ -116,6 +133,9 @@ define('action', group=ActionGroup,
        type=str, default=None)
 
 
+client = httpclient.AsyncHTTPClient()
+
+
 def check_options():
     try:
         if options.action not in POSSIBLE_ACTIONS:
@@ -131,58 +151,61 @@ def check_options():
 
 
 def gen_task(task):
-    return msgpack.packb({"task": task,
-                          "API": API_VERSION,
-                          "username": options.username})
+    return json.dumps({"task": task,
+                       "API": API_VERSION,
+                       "username": options.username})
 
 
 @Dispatcher.attach("upload")
+@jail
 def upload():
     task = {"appname": options.appname,
             "version": options.version,
-            "url": options.url}
+            "url": options.url,
+            "manifest": options.manifest}
 
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        printer = Printer()
-        map(printer.log, s.enqueue("upload",
-                                   gen_task(task)))
-    except Exception as err:
-        log.error(err)
-        exit(1)
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("upload"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("start")
+@jail
 def start():
     task = {"appname": options.appname,
             "version": options.version}
     if options.profile:
         task["profilename"] = options.profile
 
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        printer = Printer()
-        map(printer.log, s.enqueue("start", gen_task(task)))
-    except Exception as err:
-        log.error(err)
-        exit(1)
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("start"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("stop")
+@jail
 def stop():
     task = {"appname": options.appname,
             "version": options.version}
 
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        printer = Printer()
-        map(printer.log, s.enqueue("stop", gen_task(task)))
-    except Exception as err:
-        log.error(err)
-        exit(1)
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("stop"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("deploy")
+@jail
 def deploy():
     log.info("Application %s would be deployed with profile %s."
              " Its weight would be %d",
@@ -193,93 +216,103 @@ def deploy():
             "profilename": options.profile,
             "weight": options.weight}
 
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        printer = Printer()
-        map(printer.log, s.enqueue("deploy", gen_task(task)))
-    except Exception as err:
-        log.error(err.msg)
-        exit(1)
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("deploy"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("undeploy")
+@jail
 def undeploy():
     log.info("Application %s would be undeployed with profile.",
              options.appname)
     task = {"appname": options.appname,
             "version": options.version}
 
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        printer = Printer()
-        map(printer.log, s.enqueue("undeploy", gen_task(task)))
-    except Exception as err:
-        log.error(err.msg)
-        exit(1)
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("undeploy"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("remove")
+@jail
 def remove():
     log.info("Application %s would be undeployed with profile.",
              options.appname)
     task = {"appname": options.appname,
             "version": options.version}
 
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        printer = Printer()
-        map(printer.log, s.enqueue("remove", gen_task(task)))
-    except Exception as err:
-        log.error(err.msg)
-        exit(1)
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("remove"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("listing")
+@jail
 def listing():
     log.info("Fetching a list of your applications according to wildcard %s",
              options.wildcard)
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        task = {}
-        if options.wildcard:
-            task["wildcard"] = options.wildcard
-        apps = s.enqueue("list",
-                         gen_task(task)).get()
+    task = {}
+    if options.wildcard:
+        task["wildcard"] = options.wildcard
+
+    def format_reply(body):
+        try:
+            apps = json.loads(body)
+        except Exception:
+            log.info("%s", body)
+            return
+
         if len(apps) == 0:
-            print('There are no applications your username')
+            print('There are no applications with your username')
             return
-        # ToDo: Make it pretty
         for app, status in apps.iteritems():
             user, appname = app.split('_', 1)
             appname, version = appname.rsplit('_', 1)
             print(user, '\t', appname, '\t\t', version, '\t\t', status)
-    except Exception as err:
-        log.error(err)
-        exit(1)
+
+    task = gen_task(task)
+    req = HTTPRequest(gen_url("listing"),
+                      method="POST",
+                      streaming_callback=format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
 
 
 @Dispatcher.attach("info")
+@jail
 def info():
     log.info("Fetching a list of your applications")
-    try:
-        s = Service("flow@main", host=options.host, port=options.port)
-        listing0 = s.enqueue("info", gen_task(None)).get()
-        print(json.dumps(listing0, indent=2))
-    except Exception as err:
-        log.error(err)
-        exit(1)
+    task = gen_task(None)
+    req = HTTPRequest(gen_url("info"),
+                      method="POST",
+                      streaming_callback=default_format_reply,
+                      allow_ipv6=True,
+                      body=task)
+    run_request(req)
+
+
+CONF_CANDIDATES = (os.path.expanduser("~/.cocaine.conf"),
+                   ".cocaine.conf")
 
 
 def main():
-    HOME_CONFIG_PATH = os.path.expanduser("~/.cocaine.conf")
-    if os.path.exists(".cocaine.conf"):
-        CONFIG_PATH = ".cocaine.conf"
-    else:
-        CONFIG_PATH = HOME_CONFIG_PATH
-
     try:
-        if os.path.exists(CONFIG_PATH):
-            parse_config_file(CONFIG_PATH, final=False)
+        for path in filter(os.path.exists, CONF_CANDIDATES):
+            parse_config_file(path, final=False)
         options.add_parse_callback(check_options)
         parse_command_line()
     except Exception as err:
